@@ -24,7 +24,6 @@ import six
 from six.moves.urllib import request
 
 from heatclient._i18n import _
-from heatclient._i18n import _LI
 from heatclient.common import event_utils
 from heatclient.common import format_utils
 from heatclient.common import hook_utils
@@ -380,19 +379,28 @@ class ShowStack(command.ShowOne):
             metavar='<stack>',
             help='Stack to display (name or ID)',
         )
+        parser.add_argument(
+            '--no-resolve-outputs', action="store_true",
+            help=_('Do not resolve outputs of the stack.')
+        )
         return parser
 
     def take_action(self, parsed_args):
         self.log.debug("take_action(%s)", parsed_args)
 
         heat_client = self.app.client_manager.orchestration
-        return _show_stack(heat_client, stack_id=parsed_args.stack,
-                           format=parsed_args.formatter)
+        return _show_stack(
+            heat_client, stack_id=parsed_args.stack,
+            format=parsed_args.formatter,
+            resolve_outputs=(not parsed_args.no_resolve_outputs))
 
 
-def _show_stack(heat_client, stack_id, format='', short=False):
+def _show_stack(heat_client, stack_id, format='', short=False,
+                resolve_outputs=True):
     try:
-        data = heat_client.stacks.get(stack_id=stack_id)
+        _resolve_outputs = not short and resolve_outputs
+        data = heat_client.stacks.get(stack_id=stack_id,
+                                      resolve_outputs=_resolve_outputs)
     except heat_exc.HTTPNotFound:
         raise exc.CommandError('Stack not found: %s' % stack_id)
     else:
@@ -408,11 +416,10 @@ def _show_stack(heat_client, stack_id, format='', short=False):
         ]
 
         if not short:
-            columns += [
-                'parameters',
-                'outputs',
-                'links',
-            ]
+            columns.append('parameters')
+            if _resolve_outputs:
+                columns.append('outputs')
+            columns.append('links')
 
             exclude_columns = ('template_description',)
             for key in data.to_dict():
@@ -623,8 +630,6 @@ def _list(client, args=None):
             columns.pop()
         if args.long:
             columns.insert(2, 'Stack Owner')
-        if args.long or args.all_projects:
-            columns.insert(2, 'Project')
 
         if args.nested:
             columns.append('Parent')
@@ -634,6 +639,11 @@ def _list(client, args=None):
             columns.append('Deletion Time')
 
     data = client.stacks.list(**kwargs)
+    data = list(data)
+    for stk in data:
+        if hasattr(stk, 'project'):
+            columns.insert(2, 'Project')
+            break
     data = utils.sort_items(data, args.sort if args else None)
 
     return (
@@ -678,16 +688,16 @@ class DeleteStack(command.Command):
                     _("Are you sure you want to delete this stack(s) [y/N]? "))
                 prompt_response = sys.stdin.readline().lower()
                 if not prompt_response.startswith('y'):
-                    self.log.info(_LI('User did not confirm stack delete so '
-                                      'taking no action.'))
+                    self.log.info('User did not confirm stack delete so '
+                                  'taking no action.')
                     return
         except KeyboardInterrupt:  # ctrl-c
-            self.log.info(_LI('User did not confirm stack delete '
-                              '(ctrl-c) so taking no action.'))
+            self.log.info('User did not confirm stack delete '
+                          '(ctrl-c) so taking no action.')
             return
         except EOFError:  # ctrl-d
-            self.log.info(_LI('User did not confirm stack delete '
-                              '(ctrl-d) so taking no action.'))
+            self.log.info('User did not confirm stack delete '
+                          '(ctrl-d) so taking no action.')
             return
 
         failure_count = 0
@@ -863,6 +873,50 @@ class AbandonStack(format_utils.JsonFormat):
         return columns, data
 
 
+class ExportStack(format_utils.JsonFormat):
+    """Export stack data json."""
+
+    log = logging.getLogger(__name__ + '.ExportStack')
+
+    def get_parser(self, prog_name):
+        parser = super(ExportStack, self).get_parser(prog_name)
+        parser.add_argument(
+            'stack',
+            metavar='<stack>',
+            help=_('Name or ID of stack to export')
+        )
+        parser.add_argument(
+            '--output-file',
+            metavar='<output-file>',
+            help=_('File to output export data')
+        )
+
+        return parser
+
+    def take_action(self, parsed_args):
+        self.log.debug('take_action(%s)', parsed_args)
+
+        client = self.app.client_manager.orchestration
+
+        try:
+            data_info = client.stacks.export(stack_id=parsed_args.stack)
+        except heat_exc.HTTPNotFound:
+            msg = _('Stack not found: %s') % parsed_args.stack
+            raise exc.CommandError(msg)
+
+        if parsed_args.output_file is not None:
+            try:
+                with open(parsed_args.output_file, 'w') as f:
+                    f.write(jsonutils.dumps(data_info, indent=2))
+                    return [], None
+            except IOError as e:
+                raise exc.CommandError(str(e))
+
+        data = list(six.itervalues(data_info))
+        columns = list(six.iterkeys(data_info))
+        return columns, data
+
+
 class OutputShowStack(command.ShowOne):
     """Show stack output."""
 
