@@ -149,7 +149,8 @@ class TestStackCreate(TestStack):
         self.cmd.take_action(parsed_args)
 
         self.stack_client.create.assert_called_with(**self.defaults)
-        self.stack_client.get.assert_called_with(**{'stack_id': '1234'})
+        self.stack_client.get.assert_called_with(**{'stack_id': '1234',
+                                                    'resolve_outputs': False})
 
     @mock.patch('heatclient.common.event_utils.poll_for_events',
                 return_value=('CREATE_FAILED',
@@ -347,7 +348,8 @@ class TestStackUpdate(TestStack):
         self.cmd.take_action(parsed_args)
 
         self.stack_client.update.assert_called_with(**self.defaults)
-        self.stack_client.get.assert_called_with(**{'stack_id': 'my_stack'})
+        self.stack_client.get.assert_called_with(**{'stack_id': 'my_stack',
+                                                    'resolve_outputs': False})
 
     @mock.patch('heatclient.common.event_utils.poll_for_events',
                 return_value=('UPDATE_FAILED',
@@ -414,6 +416,16 @@ class TestStackShow(TestStack):
         self.cmd.take_action(parsed_args)
         self.stack_client.get.assert_called_with(**{
             'stack_id': 'my_stack',
+            'resolve_outputs': True,
+        })
+
+    def test_stack_show_explicit_no_resolve(self):
+        arglist = ['--no-resolve-outputs', '--format', self.format, 'my_stack']
+        parsed_args = self.check_parser(self.cmd, arglist, [])
+        self.cmd.take_action(parsed_args)
+        self.stack_client.get.assert_called_with(**{
+            'stack_id': 'my_stack',
+            'resolve_outputs': False,
         })
 
     def test_stack_show_short(self):
@@ -453,6 +465,9 @@ class TestStackList(TestStack):
         'deletion_time': '2015-10-21T07:50:00Z',
     }
 
+    data_with_project = copy.deepcopy(data)
+    data_with_project['project'] = 'test_project'
+
     def setUp(self):
         super(TestStackList, self).setUp()
         self.cmd = stack.ListStack(self.app, None)
@@ -495,6 +510,8 @@ class TestStackList(TestStack):
         self.assertEqual(cols, columns)
 
     def test_stack_list_all_projects(self):
+        self.stack_client.list.return_value = [
+            stacks.Stack(None, self.data_with_project)]
         kwargs = copy.deepcopy(self.defaults)
         kwargs['global_tenant'] = True
         cols = copy.deepcopy(self.columns)
@@ -507,7 +524,23 @@ class TestStackList(TestStack):
         self.stack_client.list.assert_called_with(**kwargs)
         self.assertEqual(cols, columns)
 
+    def test_stack_list_with_project(self):
+        self.stack_client.list.return_value = [
+            stacks.Stack(None, self.data_with_project)]
+        kwargs = copy.deepcopy(self.defaults)
+        cols = copy.deepcopy(self.columns)
+        cols.insert(2, 'Project')
+        arglist = []
+        parsed_args = self.check_parser(self.cmd, arglist, [])
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.stack_client.list.assert_called_with(**kwargs)
+        self.assertEqual(cols, columns)
+
     def test_stack_list_long(self):
+        self.stack_client.list.return_value = [
+            stacks.Stack(None, self.data_with_project)]
         kwargs = copy.deepcopy(self.defaults)
         kwargs['global_tenant'] = True
         cols = copy.deepcopy(self.columns)
@@ -729,7 +762,8 @@ class TestStackAdopt(TestStack):
         self.cmd.take_action(parsed_args)
 
         self.stack_client.create.assert_called_with(**self.defaults)
-        self.stack_client.get.assert_called_with(**{'stack_id': '1234'})
+        self.stack_client.get.assert_called_with(**{'stack_id': '1234',
+                                                    'resolve_outputs': False})
 
     @mock.patch('heatclient.common.event_utils.poll_for_events',
                 return_value=('ADOPT_FAILED',
@@ -743,6 +777,42 @@ class TestStackAdopt(TestStack):
         self.assertRaises(exc.CommandError, self.cmd.take_action, parsed_args)
 
 
+class TestStackExport(TestStack):
+
+    columns = ['stack_name', 'stack_status', 'id']
+    data = ['my_stack', 'ABANDONED', '1234']
+
+    response = dict(zip(columns, data))
+
+    def setUp(self):
+        super(TestStackExport, self).setUp()
+        self.cmd = stack.ExportStack(self.app, None)
+        self.stack_client.export.return_value = self.response
+
+    def test_stack_export(self):
+        arglist = ['my_stack']
+        parsed_args = self.check_parser(self.cmd, arglist, [])
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        for column in self.columns:
+            self.assertIn(column, columns)
+        for datum in self.data:
+            self.assertIn(datum, data)
+
+    @mock.patch('heatclient.osc.v1.stack.open', create=True)
+    def test_stack_export_output_file(self, mock_open):
+        arglist = ['my_stack', '--output-file', 'file.json']
+        mock_open.return_value = mock.MagicMock(spec=io.IOBase)
+        parsed_args = self.check_parser(self.cmd, arglist, [])
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        mock_open.assert_called_once_with('file.json', 'w')
+        self.assertEqual([], columns)
+        self.assertIsNone(data)
+
+
 class TestStackAbandon(TestStack):
 
     columns = ['stack_name', 'stack_status', 'id']
